; From eli@mojave.cs.cornell.edu Tue Sep 12 12:00:48 2000
; Date: 07 Sep 2000 04:01:10 -0400
; From: Eli Barzilay <eli@mojave.cs.cornell.edu>
; Newsgroups: gnu.emacs.sources
; Subject: maze.el

; Well, for increased fun, here is another cute bit of old stuff...

;           ((lambda (x) (x x)) (lambda (x) (x x)))          Eli Barzilay:
;          http://www.cs.cornell.edu/eli/meaning.html        Maze is Life!

; -------------------------------------------------------------------------------

;;; maze.el --- Maze is Life!
;;-----------------------------------------------------------------------------
;; Written by Eli Barzilay: Maze is Life!   (eli@cs.cornell.edu)

;;; Commentary:
;; A simple maze game for Emacs

;;; Code:
(defvar maze-x-pos  nil)
(defvar maze-y-pos  nil)
(defvar maze-size-x nil)
(defvar maze-size-y nil)
(defvar maze-buffer nil)

(defun maze (size-x &optional size-y)
  "Start a maze, with an optional square of SIZE-X or SIZE-X by SIZE-Y."
  (interactive "P")
  (setq maze-buffer (get-buffer-create "*** Eli Barzilay: Maze is Life! ***"))
  (switch-to-buffer maze-buffer)
  (buffer-disable-undo (current-buffer))
  (toggle-read-only 0)
  (delete-region (point-min) (point-max))
  (delete-other-windows)
  (if (null size-x)
    (setq size-x (1- (/ (window-width) 2))
          size-y (1- (/ (window-height) 2))))
  (if (null size-y) (setq size-y size-x))
  (setq maze-size-x size-x)
  (setq maze-size-y size-y)
  (message "Making Maze...")
  (maze-draw-screen)
  (maze-draw-maze)
  (switch-to-buffer maze-buffer)
  (toggle-read-only 1)
  (while (input-pending-p) (read-event))
  (overwrite-mode 1)
  (maze-mode)
  (goto-char (maze-point (1- maze-size-x) (1- maze-size-y)))
  (maze-plot-char ?X)
  (if window-system
    (progn
      (toggle-read-only 0)
      (set-regexp-face "[+-|]+" '*/green)
      (set-regexp-face "X" '*/red)
      (lookup-face-create 'yellow/red4-bold)
      (toggle-read-only 1)))
  (setq maze-x-pos 0)
  (setq maze-y-pos 0)
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?O)
  (message "Now lets see you..."))

(defvar maze-mode-map nil)

(defun maze-mode ()
  "The maze mode."
  (interactive)
  (setq major-mode 'maze-mode)
  (setq mode-name "Maze")
  (use-local-map maze-mode-map))

(or maze-mode-map
  (let ((map (make-sparse-keymap)))
    (define-key map "r"     'maze)
    (define-key map "q"     'maze-quit)
    (define-key map "h"     'maze-move-left)
    (define-key map "l"     'maze-move-right)
    (define-key map "j"     'maze-move-down)
    (define-key map "k"     'maze-move-up)
    (define-key map "\C-n"  'maze-move-down)
    (define-key map "\C-p"  'maze-move-up)
    (define-key map "\C-f"  'maze-move-right)
    (define-key map "\C-b"  'maze-move-left)
    (define-key map [up]    'maze-move-up)
    (define-key map [down]  'maze-move-down)
    (define-key map [left]  'maze-move-left)
    (define-key map [right] 'maze-move-right)
    (setq maze-mode-map map)))

(defun maze-move-up ()
  "Move up one cell."
  (interactive)
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?.)
  (goto-char (- (point) (* maze-size-x 2) 2))
  (if (memq (char-after (point)) '(?  ?.))
    (progn
      (maze-plot-char ?.)
      (setq maze-y-pos (1- maze-y-pos))))
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?O)
  (sit-for 0)
  (maze-check-win))
(defun maze-move-down ()
  "Move down one cell."
  (interactive)
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?.)
  (goto-char (+ (point) (* maze-size-x 2) 2))
  (if (memq (char-after (point)) '(?  ?.))
    (progn
      (maze-plot-char ?.)
      (setq maze-y-pos (1+ maze-y-pos))))
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?O)
  (sit-for 0)
  (maze-check-win))
(defun maze-move-left ()
  "Move left one cell."
  (interactive)
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?.)
  (goto-char (1- (point)))
  (if (memq (char-after (point)) '(?  ?.))
    (progn
      (maze-plot-char ?.)
      (setq maze-x-pos (1- maze-x-pos))))
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?O)
  (sit-for 0)
  (maze-check-win))
(defun maze-move-right ()
  "Move right one cell."
  (interactive)
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?.)
  (goto-char (1+ (point)))
  (if (memq (char-after (point)) '(?  ?.))
    (progn
      (maze-plot-char ?.)
      (setq maze-x-pos (1+ maze-x-pos))))
  (goto-char (maze-point maze-x-pos maze-y-pos))
  (maze-plot-char ?O)
  (sit-for 0)
  (maze-check-win))

(defun maze-check-win ()
  "Check if user won."
  (if (and (= maze-x-pos (1- maze-size-x)) (= maze-y-pos (1- maze-size-y)))
    (progn
      (message "**********>>> You win !!! <<<**********")
      (beep)
      (sit-for 3)
      (logo)
      (kill-buffer maze-buffer))))

(defun maze-quit ()
  "Get out."
  (interactive)
  (message "... Chicken!")
  (beep)
  (sit-for 3)
  (kill-buffer maze-buffer))

(defun maze-draw-screen ()
  "Draw the maze screen."
  (let (x y line1 line2)
    (switch-to-buffer maze-buffer)
    (setq x maze-size-x)
    (while (> x 0) (insert "+-") (setq x (1- x)))
    (insert "+\n")
    (setq line1 (buffer-substring (point-min) (point-max)))
    (setq x maze-size-x)
    (save-excursion
      (while (> x 0) (insert "|~") (setq x (1- x)))
      (insert "|\n"))
    (setq line2 (buffer-substring (point) (point-max)))
    (goto-char (point-max))
    (insert line1)
    (setq y maze-size-y)
    (while (> y 1)
      (insert line2 line1)
      (setq y (1- y)))
    (goto-char (point-min))))

(defun maze-draw-maze ()
  "Draw a maze."
  (let ((x (/ maze-size-x 2)) (y (/ maze-size-y 2))
        newx newy places dir dirs none)
    (switch-to-buffer maze-buffer)
    (goto-char (maze-point x y))
    (maze-plot-char ?#)
    (setq places (list (cons x y)))
    (while places
      (setq dirs (list '(0 . -1) '(-1 . 0) '(0 . 1) '(1 . 0)))
      (setq none t)
      (while (and dirs none)
        (setq dir (elt dirs (random (length dirs))))
        (setq dirs (delete dir dirs))
        (setq newx (+ x (car dir)))
        (setq newy (+ y (cdr dir)))
        (setq none (not (equal (char-after (maze-point newx newy)) ?~))))
      (if none
        (progn
          (maze-plot-char ? ) (sit-for 0)
          (setq places (cdr places))
          (if places
            (progn (setq x (car (car places))) (setq y (cdr (car places)))))
          (goto-char (maze-point x y)))
        (progn
          (cond
           ((> newx x) (goto-char (1+ (point)))
                       (maze-plot-char ? ))
           ((> newy y) (goto-char (+ (point) (* maze-size-x 2) 2))
                       (maze-plot-char ? ))
           ((< newx x) (goto-char (1- (point)))
                       (maze-plot-char ? ))
           ((< newy y) (goto-char (- (point) (* maze-size-x 2) 2))
                       (maze-plot-char ? )))
          (setq x newx)
          (setq y newy)
          (setq places (cons (cons x y) places))
          (goto-char (maze-point x y))
          (maze-plot-char ?#) (sit-for 0))))
    (goto-char (maze-point x y))
    (maze-plot-char ? )))

(defun maze-plot-char (c)
  "Put one char C on the screen."
  (let ((inhibit-read-only t) (inhibit-point-motion-hooks t))
    (insert-and-inherit c)
    (and window-system (eq c ?.)
         (put-text-property (1- (point)) (point) 'face 'yellow/red4-bold))
    (delete-char 1)
    (goto-char (1- (point)))))

(defun maze-point (x y)
  "Calculate the location of the X, Y point."
  (+ (* (* (1+ maze-size-x) 2) (1+ (* y 2))) (* x 2) 2))


(provide 'maze)

;;; maze.el ends here
; From eli@mojave.cs.cornell.edu Tue Sep 12 12:00:52 2000
; Date: 07 Sep 2000 12:53:52 -0400
; From: Eli Barzilay <eli@mojave.cs.cornell.edu>
; Newsgroups: gnu.emacs.sources
; Subject: Re: maze.el

; Eli Barzilay <eli@mojave.cs.cornell.edu> writes:
; > Well, for increased fun, here is another cute bit of old stuff...

; In my haste, I forgot that it uses my private coloring stuff, so one
; option is to simply remove the set-regexp-face, another is to hack
; something yourself, and the third option is posting yet another piece
; of code...  It is posted as it is, I know that it's old and in
; desperate need of a facelift.

;           ((lambda (x) (x x)) (lambda (x) (x x)))          Eli Barzilay:
;          http://www.cs.cornell.edu/eli/meaning.html        Maze is Life!

; -------------------------------------------------------------------------------
; ;; This is stolen from hilit-lookup-face-create.  I modified it to be a bit
; ;; more convenient - use "#" and "h-" instead of "hex-" and "*" instead of
; ;; "default".

(defun lookup-face-create (face &optional force)
  "Get a FACE, or create it if it doesn't exist.
In order for it to properly create the face, the following naming convention
must be used:
    (fgcolor[/bgcolor])[-bold][-italic][-underline]
Example: (lookup-face-create yellow/blue4-bold) might create and return an
appropriate face.

Each color is either the name of an X color (see .../X11/lib/X11/rgb.txt),
a hexadecimal specification of the form \"hex-[0-9A-Fa-f]+\" (\"hex-\" can be
replaced by \"h-\" or \"#\"), or \"default\".

An optional argument, FORCE, will cause the face to be recopied from the
default...which is probably of use only if you've changed fonts."

  ;;Eli: if no colors - return default face.
  (if (not window-system)
    'default

    ;; make the face if we need to...
    (let* ((fn       (symbol-name face))
           (frame    (selected-frame))
           (basefont (cdr (assq 'font (frame-parameters frame))))
           error fgcolor bgcolor)
      (cond
        ((or (null face) (memq face (face-list)))
         ;; do nothing if the face is nil or if it's predefined.
         )
        ((or force
             (not (memq face (face-list)))
             (not (string= (get face 'basefont) basefont)))
         (copy-face 'default 'scratch-face)
         ;; parse foreground color
         (if (string-match "^\\(hex-\\|h-\\|#\\)?\\([A-Za-z0-9*]+\\)" fn)
           (setq fgcolor (concat
                          (if (match-beginning 1) "#")
                          (substring fn (match-beginning 2) (match-end 2)))
                 fn (substring fn (match-end 0)))
           (error "Bad face name %S" face))
         ;; parse background color
         (if (string-match "^/\\(hex-\\|h-\\|#\\)?\\([A-Za-z0-9*]+\\)" fn)
           (setq bgcolor (concat
                          (if (match-beginning 1) "#")
                          (substring fn (match-beginning 2) (match-end 2)))
                 fn (substring fn (match-end 0))))
         (if (or (string= "*" fgcolor) (string= "default" fgcolor))
           (setq fgcolor nil))
         (if (or (string= "*" bgcolor) (string= "default" bgcolor))
           (setq bgcolor nil))
         ;; catch errors if we can't allocate the color(s)
         (condition-case nil
             (progn (set-face-foreground 'scratch-face fgcolor)
                    (set-face-background 'scratch-face bgcolor)
                    (copy-face 'scratch-face face)
                    (put face 'basefont basefont))
           (error (message "couldn't allocate color for '%s'"
                           (symbol-name face))
                  (setq face 'default)
                  (setq error t)))
         (if (not error)
           ;; don't bother w/ bold or italic if we didn't get the color
           ;; we wanted, but ignore errors making the face bold or italic
           ;; if the font isn't available, there's nothing to do about it...
           (progn
             (set-face-font face nil frame)
             (set-face-underline-p face (string-match "underline" fn))
             (if (string-match "-bold" fn)
               ;; make face bold in all frames
               (make-face-bold face nil 'noerr))
             (if (string-match "-italic" fn)
               ;; make face italic in all frames
               (make-face-italic face nil 'noerr))
             ))))
      face)))

;; This is a cheap small function - not using overlays or removing faces etc.
;; A better version may be facemenu-add-face or hilit-region-set-face.
(defun set-region-face (beg end face &optional no-create)
  "Set the face of a region.
Set the region from BEG to END to have face FACE.  `lookup-face-create' is used
to create the face, unless NO-CREATE is t."
  (and
   window-system
   (let (;; These are taken from font-lock.el - make sure that we modify the
         ;; buffer with absolutely no trace.
         (modified (buffer-modified-p))
         (buffer-undo-list t)
         before-change-functions after-change-functions
         buffer-file-name buffer-file-truename)
     (put-text-property beg end 'face
                        (if no-create face (lookup-face-create face)))
     (and (not modified) (buffer-modified-p) (set-buffer-modified-p nil)))))
;; Using the above function.
(defun set-regexp-face (regexp face &optional subexp)
  "Set the face of all occurrences of REGEXP to FACE.
Optional argument SUBEXP is either a subexpression to highlight, or a list of
pairs of subexp number and its color: (SUB . COLOR)."
  (and window-system
       (let ((face (and face (lookup-face-create face))))
         (if (consp subexp)
           (let ((p subexp))
             (while p
               (setcdr (car p) (lookup-face-create (cdr (car p))))
               (setq p (cdr p)))
             (and face (setq subexp (cons (cons 0 face) subexp)))))
         (save-excursion
           (goto-char (point-min))
           (while (re-search-forward regexp nil t nil)
             (cond
               ((consp subexp)
                (let ((p subexp))
                  (while p
                    (set-region-face (match-end (car (car p)))
                                     (match-beginning (car (car p)))
                                     (cdr (car p)) t)
                    (setq p (cdr p)))))
               (face
                (set-region-face (match-end (or subexp 0))
                                 (match-beginning (or subexp 0))
                                 face t))))))))

(defun logo ()
  (princ "YOU WON!")
  (ding)
  (ding)
  (ding)
  (ding)
)