;;; ifdef region code

;;   (autoload 'ifdef:ifdef-region "ifdef" "ifdef your code" t)
;;   (autoload 'ifdef:ifndef-region "ifdef" "ifdef your code" t)
;;   (autoload 'ifdef:ifdef-else-region "ifdef" "ifdef your code" t)
;;   (autoload 'ifdef:if-region "ifdef" "ifdef your code" t)
;;   (autoload 'ifdef:if-else-region "ifdef" "ifdef your code" t))



(defvar ifdef:last-macro nil
  "last macro name given, used as default.")

(defun ifdef:wrap-region (cpp mac &optional do-else)
  (let* ((b (region-beginning))
         (e (region-end))
         (b (progn (goto-char b) (beginning-of-line) (point)))
         (e (progn (goto-char e)
                   (if (bolp)
                       (point)
                     (progn (beginning-of-line) (forward-line 1) (point)))))
         (region-text (buffer-substring b e)))
    (delete-region b e)
    (insert (concat cpp " " mac "\n"))
    (insert region-text)
    (if do-else
        (progn (insert (concat "#else /*" mac "*/\n"))
               (insert region-text)))
    (insert (concat "#endif /*" mac "*/\n"))))

(defun ifdef:read-macro (prompt)
  (let* ((query (concat prompt (if ifdef:last-macro
                                   (concat "(" ifdef:last-macro "): ")
                                 ": ")))
         (response (read-string query)))
    (if (> (length response) 0)
        (setq ifdef:last-macro response)
      ifdef:last-macro)))

(defun ifdef:ifdef-region ()
  (interactive)
  (let ((macro (ifdef:read-macro "IFDEF region with MACRO")))
    (ifdef:wrap-region "#ifdef" macro)))

(defun ifdef:ifndef-region ()
  (interactive)
  (let ((macro (ifdef:read-macro "IFNDEF region with MACRO")))
    (ifdef:wrap-region "#ifndef" macro)))

(defun ifdef:ifdef-else-region ()
  (interactive)
  (let ((macro (ifdef:read-macro "IFDEF/ELSE region with MACRO")))
    (ifdef:wrap-region "#ifdef" macro t)))

(defun ifdef:if-region ()
  (interactive)
  (let ((macro (ifdef:read-macro "IF region with MACRO")))
    (ifdef:wrap-region "#if" macro)))

(defun ifdef:if-else-region ()
  (interactive)
  (let ((macro (ifdef:read-macro "IF/ELSE region with MACRO")))
    (ifdef:wrap-region "#if" macro t)))


(provide 'my-ifdef)
